@@ -44,6 +44,7 @@ from barman.compression import CompressionManager
 from barman.config import BackupOptions
 from barman.exceptions import (
     AbortedRetryHookScript,
+    BackupException,
     CompressionIncompatibility,
     LockFileBusy,
     SshCommandException,
@@ -360,8 +361,8 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
         Get the id of the latest/last backup in the catalog (if exists)
 
         :param status_filter: The status of the backup to return,
-            default to DEFAULT_STATUS_FILTER.
-        :return string|None: ID of the backup
+            default to :attr:`DEFAULT_STATUS_FILTER`.
+        :return str|None: ID of the backup
         """
         available_backups = self.get_available_backups(status_filter)
         if len(available_backups) == 0:
@@ -370,6 +371,29 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
         ids = sorted(available_backups.keys())
         return ids[-1]
 
+    def get_last_full_backup_id(self, status_filter=DEFAULT_STATUS_FILTER):
+        """
+        Get the id of the latest/last FULL backup in the catalog (if exists)
+
+        :param status_filter: The status of the backup to return,
+            default to :attr:`DEFAULT_STATUS_FILTER`.
+        :return str|None: ID of the backup
+        """
+        available_full_backups = list(
+            filter(
+                lambda backup: backup.is_full_and_eligible_for_incremental(),
+                self.get_available_backups(status_filter).values(),
+            )
+        )
+
+        if len(available_full_backups) == 0:
+            return None
+
+        backup_infos = sorted(
+            available_full_backups, key=lambda backup_info: backup_info.backup_id
+        )
+        return backup_infos[-1].backup_id
+
     def get_first_backup_id(self, status_filter=DEFAULT_STATUS_FILTER):
         """
         Get the id of the oldest/first backup in the catalog (if exists)
@@ -435,33 +459,6 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
           backups.
         :return bool: True if deleted, False if could not delete the backup
         """
-        if self.should_keep_backup(backup.backup_id):
-            output.warning(
-                "Skipping delete of backup %s for server %s "
-                "as it has a current keep request. If you really "
-                "want to delete this backup please remove the keep "
-                "and try again.",
-                backup.backup_id,
-                self.config.name,
-            )
-            return False
-        available_backups = self.get_available_backups(status_filter=(BackupInfo.DONE,))
-        minimum_redundancy = self.server.config.minimum_redundancy
-        # Honour minimum required redundancy
-        if backup.status == BackupInfo.DONE and minimum_redundancy >= len(
-            available_backups
-        ):
-            output.warning(
-                "Skipping delete of backup %s for server %s "
-                "due to minimum redundancy requirements "
-                "(minimum redundancy = %s, "
-                "current redundancy = %s)",
-                backup.backup_id,
-                self.config.name,
-                minimum_redundancy,
-                len(available_backups),
-            )
-            return False
         # Keep track of when the delete operation started.
         delete_start_time = datetime.datetime.now()
 
@@ -529,6 +526,7 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
                 remove_until, timelines_to_protect, wal_ranges_to_protect
             ):
                 output.info("\t%s", name)
+
         # As last action, remove the backup directory,
         # ending the delete operation
         try:
@@ -553,6 +551,30 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
             human_readable_timedelta(delete_end_time - delete_start_time),
         )
 
+        # remove its reference from its parent if it is an incremental backup
+        parent_backup = backup.get_parent_backup_info()
+        if parent_backup:
+            parent_backup.children_backup_ids.remove(backup.backup_id)
+            if not parent_backup.children_backup_ids:
+                parent_backup.children_backup_ids = None
+            parent_backup.save()
+
+        # rsync backups can have deduplication at filesystem level by using
+        # "reuse_backup = link". The deduplication size is calculated at the
+        # time the backup is taken. If we remove a backup, it may be the case
+        # that the next backup in the catalog is a rsync backup which was taken
+        # with the "link" option. With that possibility in mind, we re-calculate the
+        # deduplicated size of the next rsync backup because the removal of the
+        # previous backup can impact on that number.
+        # Note: we have no straight forward way of identifying if the next rsync
+        # backup in the catalog was taken with "link" or not because
+        # "reuse_backup" value is not stored in the "backup.info" file. In any
+        # case, the "re-calculation" can still be performed even if "link" was not
+        # used, and the only drawback is that we will waste some (small) amount
+        # of CPU/disk usage.
+        if next_backup and next_backup.backup_type == "rsync":
+            self._set_backup_sizes(next_backup)
+
         # Remove the sync lockfile if exists
         sync_lock = ServerBackupSyncLock(
             self.config.barman_lock_directory, self.config.name, backup.backup_id
@@ -584,13 +606,107 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
 
         return True
 
-    def backup(self, wait=False, wait_timeout=None, name=None):
+    def _set_backup_sizes(self, backup_info, fsync=False):
+        """
+        Set the actual size on disk of a backup.
+
+        Optionally fsync all files in the backup.
+
+        :param LocalBackupInfo backup_info: the backup to update
+        :param bool fsync: whether to fsync files to disk
+        """
+        backup_size = 0
+        deduplicated_size = 0
+        backup_dest = backup_info.get_basebackup_directory()
+        for dir_path, _, file_names in os.walk(backup_dest):
+            if fsync:
+                # If fsync, execute fsync() on the containing directory
+                fsync_dir(dir_path)
+            for filename in file_names:
+                file_path = os.path.join(dir_path, filename)
+                # If fsync, execute fsync() on all the contained files
+                file_stat = fsync_file(file_path) if fsync else os.stat(file_path)
+                backup_size += file_stat.st_size
+                # Excludes hard links from real backup size and only counts
+                # unique files for deduplicated size
+                if file_stat.st_nlink == 1:
+                    deduplicated_size += file_stat.st_size
+        # Save size into BackupInfo object
+        backup_info.set_attribute("size", backup_size)
+        backup_info.set_attribute("deduplicated_size", deduplicated_size)
+        backup_info.save()
+
+    def validate_backup_args(self, **kwargs):
+        """
+        Validate backup arguments and Postgres configurations. Arguments
+        might be syntactically correct but still be invalid if necessary
+        Postgres configurations are not met.
+
+        :kwparam str parent_backup_id: id of the parent backup when taking a
+            Postgres incremental backup
+        :raises BackupException: if a command argument is considered invalid
+        """
+        if "parent_backup_id" in kwargs:
+            self._validate_incremental_backup_configs(**kwargs)
+
+    def _validate_incremental_backup_configs(self, **kwargs):
+        """
+        Check required configurations for a Postgres incremental backup
+
+        :raises BackupException: if a required configuration is missing
+        """
+        if self.server.postgres.server_version < 170000:
+            raise BackupException(
+                "Postgres version 17 or greater is required for incremental backups "
+                "using the Postgres backup method"
+            )
+
+        if self.config.backup_method != "postgres":
+            raise BackupException(
+                "Backup using the `--incremental` flag is available only for "
+                "'backup_method = postgres'. Check Barman's documentation for "
+                "more help on this topic."
+            )
+
+        summarize_wal = self.server.postgres.get_setting("summarize_wal")
+        if summarize_wal != "on":
+            raise BackupException(
+                "'summarize_wal' option has to be enabled in the Postgres server "
+                "to perform an incremental backup using the Postgres backup method"
+            )
+
+        if self.config.backup_compression is not None:
+            raise BackupException(
+                "Incremental backups cannot be taken with "
+                "'backup_compression' set in the configuration options."
+            )
+
+        parent_backup_id = kwargs.get("parent_backup_id")
+        parent_backup_info = self.get_backup(parent_backup_id)
+
+        if parent_backup_info:
+            if parent_backup_info.summarize_wal != "on":
+                raise BackupException(
+                    "The specified backup is not eligible as a parent for an "
+                    "incremental backup because WAL summaries were not enabled "
+                    "when that backup was taken."
+                )
+            if parent_backup_info.compression is not None:
+                raise BackupException(
+                    "The specified backup cannot be a parent for an "
+                    "incremental backup. Reason: "
+                    "Compressed backups are not eligible as parents of incremental backups."
+                )
+
+    def backup(self, wait=False, wait_timeout=None, name=None, **kwargs):
         """
         Performs a backup for the server
 
         :param bool wait: wait for all the required WAL files to be archived
         :param int|None wait_timeout:
         :param str|None name: the friendly name to be saved with this backup
+        :kwparam str parent_backup_id: id of the parent backup when taking a
+            Postgres incremental backup
         :return BackupInfo: the generated BackupInfo
         """
         _logger.debug("initialising backup information")
@@ -605,6 +721,11 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
             )
             backup_info.set_attribute("systemid", self.server.systemid)
 
+            backup_info.set_attribute(
+                "parent_backup_id",
+                kwargs.get("parent_backup_id"),
+            )
+
             backup_info.save()
             self.backup_cache_add(backup_info)
             output.info(
@@ -694,6 +815,22 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
 
         finally:
             if backup_info:
+                # IF is an incremental backup, we save here child backup info id
+                # inside the parent list of children. no matter if the backup
+                # is successful or not. This is needed to be able to retrieve
+                # also failed incremental backups for removal or other operations
+                # like show-backup.
+                parent_backup_info = backup_info.get_parent_backup_info()
+
+                if parent_backup_info:
+                    if parent_backup_info.children_backup_ids:
+                        parent_backup_info.children_backup_ids.append(  # type: ignore
+                            backup_info.backup_id
+                        )
+                    else:
+                        parent_backup_info.children_backup_ids = [backup_info.backup_id]
+                    parent_backup_info.save()
+
                 backup_info.save()
 
                 # Make sure we are not holding any PostgreSQL connection
@@ -1379,23 +1516,10 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
         # Calculate the base backup size
         self.executor.current_action = "calculating backup size"
         _logger.debug(self.executor.current_action)
-        backup_size = 0
-        deduplicated_size = 0
-        backup_dest = backup_info.get_basebackup_directory()
-        for dir_path, _, file_names in os.walk(backup_dest):
-            # execute fsync() on the containing directory
-            fsync_dir(dir_path)
-            # execute fsync() on all the contained files
-            for filename in file_names:
-                file_path = os.path.join(dir_path, filename)
-                file_stat = fsync_file(file_path)
-                backup_size += file_stat.st_size
-                # Excludes hard links from real backup size
-                if file_stat.st_nlink == 1:
-                    deduplicated_size += file_stat.st_size
-        # Save size into BackupInfo object
-        backup_info.set_attribute("size", backup_size)
-        backup_info.set_attribute("deduplicated_size", deduplicated_size)
+        # Set backup sizes with fsync. We need to fsync files here to make sure
+        # the backup files are persisted to disk, so we don't lose the backup in
+        # the event of a system crash.
+        self._set_backup_sizes(backup_info, fsync=True)
         if backup_info.size > 0:
             deduplication_ratio = 1 - (
                 float(backup_info.deduplicated_size) / backup_info.size
@@ -1487,6 +1611,10 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
             )
             backup_info.status = BackupInfo.FAILED
             backup_info.save()
+            output.error(
+                "This backup has been marked as FAILED due to the "
+                "following reason: %s" % backup_info.error
+            )
             return
 
         if end_wal <= last_archived_wal:
