@@ -52,7 +52,7 @@ from barman.annotations import KeepManager
 from barman.config import (
     ConfigChangesProcessor,
     RecoveryOptions,
-    parse_recovery_staging_path,
+    parse_staging_path,
 )
 from barman.exceptions import (
     BadXlogSegmentName,
@@ -406,6 +406,13 @@ def backup_completer(prefix, parsed_args, **kwargs):
             action="store_false",
             default=SUPPRESS,
         ),
+        argument(
+            "--incremental",
+            completer=backup_completer,
+            dest="backup_id",
+            help="performs an incremental backup. An ID of a previous backup must "
+            "be provided ('latest' and 'latest-full' are also available options)",
+        ),
         argument(
             "--reuse-backup",
             nargs="?",
@@ -477,6 +484,13 @@ def backup_completer(prefix, parsed_args, **kwargs):
             default=None,
             type=check_non_negative,
         ),
+        argument(
+            "--keepalive-interval",
+            help="An interval, in seconds, at which a heartbeat query will be sent "
+            "to the server to keep the libpq connection alive during an Rsync backup.",
+            dest="keepalive_interval",
+            type=check_non_negative,
+        ),
         argument(
             "--name",
             help="a name which can be used to reference this backup in barman "
@@ -515,12 +529,20 @@ def backup(args):
         if not manage_server_command(server, name):
             continue
 
+        incremental_kwargs = {}
+
+        if args.backup_id is not None:
+            parent_backup_info = parse_backup_id(server, args)
+            if parent_backup_info:
+                incremental_kwargs["parent_backup_id"] = parent_backup_info.backup_id
         if args.reuse_backup is not None:
             server.config.reuse_backup = args.reuse_backup
         if args.retry_sleep is not None:
             server.config.basebackup_retry_sleep = args.retry_sleep
         if args.retry_times is not None:
             server.config.basebackup_retry_times = args.retry_times
+        if args.keepalive_interval is not None:
+            server.config.keepalive_interval = args.keepalive_interval
         if hasattr(args, "immediate_checkpoint"):
             # As well as overriding the immediate_checkpoint value in the config
             # we must also update the immediate_checkpoint attribute on the
@@ -547,6 +569,7 @@ def backup(args):
                 wait=args.wait,
                 wait_timeout=args.wait_timeout,
                 backup_name=args.backup_name,
+                **incremental_kwargs,
             )
     output.close_and_exit()
 
@@ -843,6 +866,16 @@ def rebuild_xlogdb(args):
                 "backup."
             ),
         ),
+        argument(
+            "--local-staging-path",
+            help=(
+                "A path to a location on the local host where incremental backups "
+                "will be combined during the recovery. This location must have "
+                "enough available space to temporarily hold the new synthetic "
+                "backup. This option is *required* when recovering from an "
+                "incremental backup."
+            ),
+        ),
         argument(
             "--recovery-conf-filename",
             dest="recovery_conf_filename",
@@ -901,9 +934,7 @@ def recover(args):
         # Set the recovery staging path from the cli if it is set
         if args.recovery_staging_path is not None:
             try:
-                recovery_staging_path = parse_recovery_staging_path(
-                    args.recovery_staging_path
-                )
+                recovery_staging_path = parse_staging_path(args.recovery_staging_path)
             except ValueError as exc:
                 output.error("Cannot parse recovery staging path: %s", str(exc))
                 output.close_and_exit()
@@ -924,6 +955,32 @@ def recover(args):
             )
             output.close_and_exit()
 
+    # If the backup to be recovered is incremental then there are additional
+    # checks to be carried out
+    if backup_id.is_incremental:
+        # Set the local staging path from the cli if it is set
+        if args.local_staging_path is not None:
+            try:
+                local_staging_path = parse_staging_path(args.local_staging_path)
+            except ValueError as exc:
+                output.error("Cannot parse local staging path: %s", str(exc))
+                output.close_and_exit()
+            server.config.local_staging_path = local_staging_path
+        # If the backup is incremental but there is no local_staging_path
+        # then this is an error - the user *must* tell barman where recovery
+        # data can be staged.
+        if server.config.local_staging_path is None:
+            output.error(
+                "Cannot recover from backup '%s' of server '%s': "
+                "backup will be combined with pg_combinebackup in the "
+                "barman host but no local staging path is provided. "
+                "Either set local_staging_path in the Barman config "
+                "or use the --local-staging-path argument.",
+                args.backup_id,
+                server.config.name,
+            )
+            output.close_and_exit()
+
     # decode the tablespace relocation rules
     tablespaces = {}
     if args.tablespace:
@@ -1086,7 +1143,7 @@ def recover(args):
                 target_action=getattr(args, "target_action", None),
                 standby_mode=getattr(args, "standby_mode", None),
                 recovery_conf_filename=args.recovery_conf_filename,
-                **snapshot_kwargs
+                **snapshot_kwargs,
             )
         except RecoveryException as exc:
             output.error(force_str(exc))
@@ -1806,6 +1863,13 @@ def keep(args):
             ) % (backup_info.backup_id, backup_info.status)
             output.error(msg)
             output.close_and_exit()
+        if backup_info.is_incremental:
+            msg = (
+                "Unable to execute the keep command on backup %s: is an incremental backup.\n"
+                "Only full backups are eligible for the use of the keep command."
+            ) % (backup_info.backup_id)
+            output.error(msg)
+            output.close_and_exit()
         backup_manager.keep_backup(backup_info.backup_id, args.target)
 
 
