@@ -197,7 +197,12 @@ def get_snapshot_interface(config):
     elif config.cloud_provider == "aws-s3":
         from barman.cloud_providers.aws_s3 import AwsCloudSnapshotInterface
 
-        return AwsCloudSnapshotInterface(config.aws_profile, config.aws_region)
+        args = [
+            config.aws_profile,
+            config.aws_region,
+            config.aws_await_snapshots_timeout,
+        ]
+        return AwsCloudSnapshotInterface(*args)
     else:
         raise CloudProviderUnsupported(
             "No snapshot provider for cloud provider: %s" % config.cloud_provider
@@ -245,7 +250,9 @@ def get_snapshot_interface_from_server_config(server_config):
         from barman.cloud_providers.aws_s3 import AwsCloudSnapshotInterface
 
         return AwsCloudSnapshotInterface(
-            server_config.aws_profile, server_config.aws_region
+            server_config.aws_profile,
+            server_config.aws_region,
+            server_config.aws_await_snapshots_timeout,
         )
     else:
         raise CloudProviderUnsupported(
@@ -295,14 +302,17 @@ def get_snapshot_interface_from_backup_info(backup_info, config=None):
                 "backup_info has snapshot provider 'azure' but "
                 "subscription_id is not set"
             )
-        if config is not None and hasattr(config, "azure_resource_group"):
-            resource_group = config.azure_resource_group
-        else:
-            resource_group = None
+        resource_group = None
+        azure_credential = None
+        if config is not None:
+            if hasattr(config, "azure_resource_group"):
+                resource_group = config.azure_resource_group
+            if hasattr(config, "azure_credential"):
+                azure_credential = config.azure_credential
         return AzureCloudSnapshotInterface(
             backup_info.snapshots_info.subscription_id,
             resource_group=resource_group,
-            credential=_get_azure_credential(config.azure_credential),
+            credential=_get_azure_credential(azure_credential),
         )
     elif backup_info.snapshots_info.provider == "aws":
         from barman.cloud_providers.aws_s3 import AwsCloudSnapshotInterface
