@@ -359,7 +359,7 @@ def parse_backup_method(value):
     )
 
 
-def parse_recovery_staging_path(value):
+def parse_staging_path(value):
     if value is None or os.path.isabs(value):
         return value
     raise ValueError("Invalid value : '%s' (must be an absolute path)" % value)
@@ -488,6 +488,7 @@ class ServerConfig(BaseConfig):
         "archiver",
         "archiver_batch_size",
         "autogenerate_manifest",
+        "aws_await_snapshots_timeout",
         "aws_profile",
         "aws_region",
         "azure_credential",
@@ -520,9 +521,11 @@ class ServerConfig(BaseConfig):
         "gcp_zone",
         "immediate_checkpoint",
         "incoming_wals_directory",
+        "keepalive_interval",
         "last_backup_maximum_age",
         "last_backup_minimum_size",
         "last_wal_maximum_age",
+        "local_staging_path",
         "max_incoming_wals_queue",
         "minimum_redundancy",
         "network_compression",
@@ -583,6 +586,7 @@ class ServerConfig(BaseConfig):
         "archiver",
         "archiver_batch_size",
         "autogenerate_manifest",
+        "aws_await_snapshots_timeout",
         "aws_profile",
         "aws_region",
         "azure_credential",
@@ -608,9 +612,11 @@ class ServerConfig(BaseConfig):
         "forward_config_path",
         "gcp_project",
         "immediate_checkpoint",
+        "keepalive_internval",
         "last_backup_maximum_age",
         "last_backup_minimum_size",
         "last_wal_maximum_age",
+        "local_staging_path",
         "max_incoming_wals_queue",
         "minimum_redundancy",
         "network_compression",
@@ -660,6 +666,7 @@ class ServerConfig(BaseConfig):
         "archiver": "off",
         "archiver_batch_size": "0",
         "autogenerate_manifest": "false",
+        "aws_await_snapshots_timeout": "3600",
         "backup_directory": "%(barman_home)s/%(name)s",
         "backup_method": "rsync",
         "backup_options": "",
@@ -673,6 +680,7 @@ class ServerConfig(BaseConfig):
         "forward_config_path": "false",
         "immediate_checkpoint": "false",
         "incoming_wals_directory": "%(backup_directory)s/incoming",
+        "keepalive_interval": "60",
         "minimum_redundancy": "0",
         "network_compression": "false",
         "parallel_jobs": "1",
@@ -701,6 +709,7 @@ class ServerConfig(BaseConfig):
         "archiver": parse_boolean,
         "archiver_batch_size": int,
         "autogenerate_manifest": parse_boolean,
+        "aws_await_snapshots_timeout": int,
         "backup_compression": parse_backup_compression,
         "backup_compression_format": parse_backup_compression_format,
         "backup_compression_level": int,
@@ -713,10 +722,12 @@ class ServerConfig(BaseConfig):
         "check_timeout": int,
         "disabled": parse_boolean,
         "forward_config_path": parse_boolean,
+        "keepalive_interval": int,
         "immediate_checkpoint": parse_boolean,
         "last_backup_maximum_age": parse_time_interval,
         "last_backup_minimum_size": parse_si_suffix,
         "last_wal_maximum_age": parse_time_interval,
+        "local_staging_path": parse_staging_path,
         "max_incoming_wals_queue": int,
         "network_compression": parse_boolean,
         "parallel_jobs": int,
@@ -724,7 +735,7 @@ class ServerConfig(BaseConfig):
         "parallel_jobs_start_batch_size": int,
         "primary_checkpoint_timeout": int,
         "recovery_options": RecoveryOptions,
-        "recovery_staging_path": parse_recovery_staging_path,
+        "recovery_staging_path": parse_staging_path,
         "create_slot": parse_create_slot,
         "reuse_backup": parse_reuse_backup,
         "snapshot_disks": parse_snapshot_disks,
