@@ -108,6 +108,32 @@ def load_datetime_tz(time_str):
     return timestamp
 
 
+def dump_backup_ids(ids):
+    """
+    Dump a list of backup IDs to disk as a string.
+
+    :param list[str]|None ids: list of backup IDs, if any
+    :return str|None: the dumped string.
+    """
+    if ids:
+        return ",".join(ids)
+    else:
+        return None
+
+
+def load_backup_ids(string):
+    """
+    Load a list of backup IDs from disk as a :class:`list`.
+
+    :param string: the string to be loaded as a list.
+    :return list[str]|None: the list of backup IDs, if any.
+    """
+    if string:
+        return string.split(",")
+    else:
+        return None
+
+
 class Field(object):
     def __init__(self, name, dump=None, load=None, default=None, doc=None):
         """
@@ -161,7 +187,6 @@ class Field(object):
 
 
 class FieldListFile(object):
-
     __slots__ = ("_fields", "filename")
 
     # A list of fields which should be hidden if they are not set.
@@ -431,7 +456,6 @@ class WalFileInfo(FieldListFile):
 
 
 class BackupInfo(FieldListFile):
-
     #: Conversion to string
     EMPTY = "EMPTY"
     STARTED = "STARTED"
@@ -498,6 +522,16 @@ class BackupInfo(FieldListFile):
     snapshots_info = Field(
         "snapshots_info", load=load_snapshots_info, dump=output_snapshots_info
     )
+    data_checksums = Field("data_checksums")
+    summarize_wal = Field("summarize_wal")
+    parent_backup_id = Field("parent_backup_id")
+    children_backup_ids = Field(
+        "children_backup_ids",
+        dump=dump_backup_ids,
+        load=load_backup_ids,
+    )
+
+    cluster_size = Field("cluster_size", load=int)
 
     __slots__ = "backup_id", "backup_version"
 
@@ -559,6 +593,94 @@ class BackupInfo(FieldListFile):
         """
         setattr(self, key, value)
 
+    @property
+    def is_incremental(self):
+        """
+        Only checks if the backup_info is an incremental backup
+
+        .. note::
+            This property only makes sense in the context of local backups stored in the
+            Barman server. However, this property is used for retention policies
+            processing, code which is shared among local and cloud backups. As this
+            property always returns ``False`` for cloud backups, it can safely be reused
+            in their code paths as well, though.
+
+        :return bool: ``True`` if this backup has a parent, ``False`` otherwise.
+        """
+        return self.parent_backup_id is not None
+
+    @property
+    def has_children(self):
+        """
+        Only checks if the backup_info has children
+
+        .. note::
+            This property only makes sense in the context of local backups stored in the
+            Barman server. However, this property is used for retention policies
+            processing, code which is shared among local and cloud backups. As this
+            property always returns ``False`` for cloud backups, it can safely be reused
+            in their code paths as well, though.
+
+        :return bool: ``True`` if this backup has at least one child, ``False`` otherwise.
+        """
+        return self.children_backup_ids is not None
+
+    @property
+    def backup_type(self):
+        """
+        Returns a string with the backup type label.
+
+        .. note::
+            Even though this property is available in this base class, it is not
+            expected to be used in the context of cloud backups.
+
+        The backup type can be one of the following:
+        - ``snapshot``: If the backup mode starts with ``snapshot``.
+        - ``rsync``: If the backup mode starts with ``rsync``.
+        - ``incremental``: If the mode is ``postgres`` and the backup is incremental.
+        - ``full``: If the mode is ``postgres`` and the backup is not incremental.
+
+        :return str: The backup type label.
+        """
+        if self.mode.startswith("snapshot"):
+            return "snapshot"
+        elif self.mode.startswith("rsync"):
+            return "rsync"
+        return "incremental" if self.is_incremental else "full"
+
+    @property
+    def deduplication_ratio(self):
+        """
+        Returns a value between and including ``0`` and ``1`` related to the estimate
+        deduplication ratio of the backup.
+
+        .. note::
+            For ``rsync`` backups, the :attr:`size` of the backup, which is the sum of
+            all file sizes in basebackup directory, is used to calculate the
+            ratio. For ``postgres`` backups, the :attr:`cluster_size` is used, which contains
+            the estimated size of the Postgres cluster at backup time.
+
+            We perform this calculation to make an estimation of how much network and disk
+            I/O has been saved when taking an incremental backup through ``rsync`` or through
+            ``pg_basebackup``.
+
+            We abuse of the term "deduplication" here. It makes more sense to ``rsync`` than to
+            ``postgres`` method. However, the idea is the same in both cases: get an estimation
+            of resources saving.
+
+        .. note::
+            Even though this property is available in this base class, it is not
+            expected to be used in the context of cloud backups.
+
+        :return float: The backup deduplication ratio.
+        """
+        size = self.cluster_size
+        if self.backup_type == "rsync":
+            size = self.size
+        if size and self.deduplicated_size:
+            return 1 - (self.deduplicated_size / size)
+        return 0
+
     def to_dict(self):
         """
         Return the backup_info content as a simple dictionary
@@ -792,3 +914,190 @@ class LocalBackupInfo(BackupInfo):
             if not os.path.exists(dir_name):
                 os.makedirs(dir_name)
         super(LocalBackupInfo, self).save(filename=filename, file_object=file_object)
+
+    def get_backup_manifest_path(self):
+        """
+        Get the full path to the backup manifest file
+
+        :return str: the full path to the backup manifest file.
+        """
+        return os.path.join(self.get_data_directory(), "backup_manifest")
+
+    def get_parent_backup_info(self):
+        """
+        If the backup is incremental, build the :class:`LocalBackupInfo` object
+        for the parent backup and return it.
+        If the backup is not incremental OR the status of the parent
+        backup is ``EMPTY``, return ``None``.
+
+        :return LocalBackupInfo|None: the parent backup info object,
+            or None if it does not exist or is empty.
+        """
+        if self.is_incremental:
+            backup_info = LocalBackupInfo(
+                self.server,
+                backup_id=self.parent_backup_id,
+            )
+
+            if backup_info.status != BackupInfo.EMPTY:
+                return backup_info
+
+        return None
+
+    def get_child_backup_info(self, child_backup_id):
+        """
+        Allow to retrieve a specific child of the current incremental backup,
+        if there are any.
+
+        If the child backup exists, the LocalBackupInfo object for it is returned.
+        If does not exist or its status is `EMPTY`, return None
+
+        :param str child_backup_id: the ID of the child backup to retrieve
+
+        :return LocalBackupInfo|None: the child backup info object,
+            or None if it does not exist or is empty.
+        """
+        if self.children_backup_ids:
+            if child_backup_id in self.children_backup_ids:
+                backup_info = LocalBackupInfo(
+                    self.server,
+                    backup_id=child_backup_id,
+                )
+
+                if backup_info.status != BackupInfo.EMPTY:
+                    return backup_info
+
+        return None
+
+    def walk_backups_tree(self, return_self=True):
+        """
+        Walk through all the children backups of the current backup.
+
+        .. note::
+            The objects are returned with a bottom-up approach, including all
+            children backups plus the caller backup.
+
+        :param bool return_self: Whether to return the current backup.
+            Default to ``True``.
+
+        :yields: a generator of :class:`LocalBackupInfo` objects for each
+            backup, walking from the leaves to self.
+        """
+
+        if self.children_backup_ids:
+            for child_backup_id in self.children_backup_ids:
+                backup_info = LocalBackupInfo(
+                    self.server,
+                    backup_id=child_backup_id,
+                )
+                yield from backup_info.walk_backups_tree()
+        if not return_self:
+            return
+        yield self
+
+    def walk_to_root(self, return_self=True):
+        """
+        Walk through all the parent backups of the current backup.
+
+        .. note::
+            The objects are returned with a bottom-up approach, including all
+            parents backups plus the caller backup if *return_self* is ``True``.
+
+        :param bool return_self: Whether to return the current backup.
+            Default to ``True``.
+
+        :yield: a generator of :class:`LocalBackupInfo` objects for each parent backup.
+        """
+
+        if return_self:
+            yield self
+        backup_info = self.get_parent_backup_info()
+        while backup_info:
+            yield backup_info
+            backup_info = backup_info.get_parent_backup_info()
+
+    def is_checksum_consistent(self):
+        """
+            Check if all backups in the chain are consistent with their checksums
+            configurations.
+
+            The backup chain is considered inconsistent if the current backup was taken with
+            ``data_checksums`` enabled and any of its ascendants were taken with it disabled.
+            It is considered consistent otherwise.
+
+        .. note::
+            While this method was created to check inconsistencies in chains of one
+            or more (Postgres 17+ core) incremental backups, it can be safely used
+            with any Postgres version and with any Barman backup method. That is
+            true because it always returns ``True`` when called for a Postgres full
+            backup or for a rsync backup.
+
+            :return bool: ``True`` if it is consistent, ``False`` otherwise.
+        """
+        if self.data_checksums != "on" or not self.is_incremental:
+            return True
+        for backup in self.walk_to_root(return_self=False):
+            if backup.data_checksums == "off":
+                return False
+        return True
+
+    def is_full_and_eligible_for_incremental(self):
+        """
+        Check if this is a full backup taken with `postgres` method and which is
+        eligible to be a parent for an incremental backup.
+
+        .. note::
+            Only consider backups which are eligible for Postgres core
+            incremental backups:
+
+            * backup_method = ``postgres``
+            * summarize_wal = ``on``
+            * is_incremental = ``False``
+
+        :return bool: True if it's a full backup or False if not.
+        """
+        if (
+            self.mode == "postgres"
+            and self.summarize_wal == "on"
+            and not self.is_incremental
+        ):
+            return True
+        return False
+
+
+class SyntheticBackupInfo(LocalBackupInfo):
+    def __init__(
+        self, server, base_directory, backup_id=None, info_file=None, **kwargs
+    ):
+        """
+        Stores meta information about a single synthetic backup.
+
+        .. note::
+            A synthetic backup is a base backup which was artificially created
+            through ``pg_combinebackup``. A synthetic backup is not part of
+            the Barman backup catalog, and only exists so we are able to
+            recover a backup created by ``pg_combinebackup`` utility, as
+            almost all functions and methods require a backup info object.
+
+        The only difference from this class to its parent :class:`LocalBackupInfo`
+        is that it accepts a custom base directory for the backup as synthetic
+        backups are expected to live on directories other than the default
+        ``<server_name>/base`` path.
+
+        :param barman.server.Server server: the server that owns the
+            synthetic backup
+        :param str base_directory: the root directory where this synthetic
+            backup resides, essentially an override to the
+            ``server.config.basebackups_directory`` configuration.
+        :param str|None backup_id: the backup id of this backup
+        :param None|str|TextIO info_file: path or file descriptor of an existing
+            synthetic ``backup.info`` file
+        """
+        self.base_directory = base_directory
+        super(SyntheticBackupInfo, self).__init__(
+            server, info_file, backup_id, **kwargs
+        )
+
+    def get_basebackup_directory(self):
+        """Get the backup directory based on its base directory"""
+        return os.path.join(self.base_directory, self.backup_id)
