@@ -21,9 +21,14 @@ This module represents the interface towards a PostgreSQL server.
 """
 
 import atexit
+import datetime
 import logging
 from abc import ABCMeta
 from multiprocessing import Process, Queue
+import os
+import signal
+import threading
+import time
 
 try:
     from queue import Empty
@@ -39,6 +44,7 @@ from barman.exceptions import (
     ConninfoException,
     PostgresAppNameError,
     PostgresConnectionError,
+    PostgresConnectionLost,
     PostgresDuplicateReplicationSlot,
     PostgresException,
     PostgresInvalidReplicationSlot,
@@ -47,7 +53,7 @@ from barman.exceptions import (
     PostgresReplicationSlotInUse,
     PostgresReplicationSlotsFull,
     BackupFunctionsAccessRequired,
-    PostgresSuperuserRequired,
+    PostgresCheckpointPrivilegesRequired,
     PostgresUnsupportedFeature,
 )
 from barman.infofile import Tablespace
@@ -456,6 +462,8 @@ class PostgreSQLConnection(PostgreSQL):
     WALSTREAMER = 2
     ANY_STREAMING_CLIENT = (STANDBY, WALSTREAMER)
 
+    HEARTBEAT_QUERY = "SELECT 1"
+
     def __init__(
         self,
         conninfo,
@@ -497,6 +505,11 @@ class PostgreSQLConnection(PostgreSQL):
                 raise PostgresAppNameError(force_str(e).strip())
         return self._conn
 
+    @property
+    def has_connection(self):
+        """Checks if the Postgres connection has already been set"""
+        return True if self._conn is not None else False
+
     @property
     def server_txt_version(self):
         """
@@ -637,6 +650,69 @@ class PostgreSQLConnection(PostgreSQL):
             )
             return None
 
+    @property
+    def has_checkpoint_privileges(self):
+        """
+        Returns true if the current user is a superuser or if,
+        for PostgreSQL 14 and above, the user has the "pg_checkpoint" role.
+        """
+
+        if self.server_version < 140000:
+            return self.is_superuser
+
+        if self.is_superuser:
+            return True
+        else:
+            role_check_query = (
+                "select pg_has_role(CURRENT_USER ,'pg_checkpoint', 'MEMBER');"
+            )
+            try:
+                cur = self._cursor()
+                cur.execute(role_check_query)
+                return cur.fetchone()[0]
+            except (PostgresConnectionError, psycopg2.Error) as e:
+                _logger.warning(
+                    "Error checking privileges for functions needed for creating checkpoints: %s",
+                    force_str(e).strip(),
+                )
+                return None
+
+    @property
+    def has_monitoring_privileges(self):
+        """
+        Check whether the current user can access monitoring information.
+
+        Returns ``True`` if the current user is a superuser or if the user has the
+        necessary privileges to monitor system status.
+
+        :rtype: bool
+        :return: ``True`` if the current user can access monitoring information.
+        """
+        if self.is_superuser:
+            return True
+        else:
+            monitoring_check_query = """
+            SELECT
+            (
+                pg_has_role(CURRENT_USER, 'pg_monitor', 'MEMBER')
+                OR
+                (
+                    pg_has_role(CURRENT_USER, 'pg_read_all_settings', 'MEMBER')
+                    AND pg_has_role(CURRENT_USER, 'pg_read_all_stats', 'MEMBER')
+                )
+            )
+            """
+            try:
+                cur = self._cursor()
+                cur.execute(monitoring_check_query)
+                return cur.fetchone()[0]
+            except (PostgresConnectionError, psycopg2.Error) as e:
+                _logger.debug(
+                    "Error checking privileges for functions needed for monitoring: %s",
+                    force_str(e).strip(),
+                )
+                return None
+
     @property
     def current_xlog_info(self):
         """
@@ -910,6 +986,7 @@ class PostgreSQLConnection(PostgreSQL):
 
             result["is_superuser"] = self.is_superuser
             result["has_backup_privileges"] = self.has_backup_privileges
+            result["has_monitoring_privileges"] = self.has_monitoring_privileges
             result["is_in_recovery"] = self.is_in_recovery
             result["server_txt_version"] = self.server_txt_version
             result["version_supported"] = self.is_minimal_postgres_version()
@@ -1350,8 +1427,8 @@ class PostgreSQLConnection(PostgreSQL):
             conn = self.connect()
 
             # Requires superuser privilege
-            if not self.is_superuser:
-                raise PostgresSuperuserRequired()
+            if not self.has_checkpoint_privileges:
+                raise PostgresCheckpointPrivilegesRequired()
 
             cur = conn.cursor()
             cur.execute("CHECKPOINT")
@@ -1365,7 +1442,7 @@ class PostgreSQLConnection(PostgreSQL):
         try:
             cur = self._cursor(cursor_factory=NamedTupleCursor)
 
-            if not self.has_backup_privileges:
+            if not self.has_monitoring_privileges:
                 raise BackupFunctionsAccessRequired(
                     "Postgres user '%s' is missing required privileges "
                     '(see "Preliminary steps" in the Barman manual)'
@@ -1537,6 +1614,26 @@ class PostgreSQLConnection(PostgreSQL):
             # the format of the synchronous_standby_names content
             return [x.strip().strip('"') for x in names_list.split(",")]
 
+    def send_heartbeat_query(self):
+        """
+        Sends a heartbeat query to the server with the already opened connection.
+
+        :returns tuple[bool, Exception|None]: A tuple where the first value is a boolean
+            indicating if the query executed successfully or not and the second is the
+            exception raised by ``psycopg2`` in case it did not succeed.
+        """
+        try:
+            with self._conn.cursor() as cursor:
+                cursor.execute(self.HEARTBEAT_QUERY)
+                _logger.debug("Sent heartbeat query to maintain the current connection")
+            return True, None
+        except psycopg2.Error as ex:
+            _logger.debug(
+                "Failed to execute heartbeat query on the current connection: %s"
+                % force_str(ex)
+            )
+            return False, ex
+
     @property
     def name_map(self):
         """
@@ -1581,6 +1678,7 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
         primary_conninfo,
         immediate_checkpoint=False,
         slot_name=None,
+        primary_checkpoint_timeout=0,
         application_name="barman",
     ):
         """
@@ -1606,6 +1704,7 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
         # The standby needs a connection to the primary so that it can
         # perform WAL switches itself when calling pg_backup_stop.
         self.primary = PostgreSQLConnection(self.primary_conninfo)
+        self.primary_checkpoint_timeout = primary_checkpoint_timeout
 
     def close(self):
         """Close the connection to PostgreSQL."""
@@ -1658,6 +1757,32 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
                 except Empty:
                     # An empty queue just means we haven't yet been told to stop
                     pass
+            if self.primary_checkpoint_timeout:
+                _logger.warning(
+                    "Barman attempted to switch WALs %s times on the primary "
+                    "server, but the backup has not yet completed. "
+                    "A checkpoint will be forced on the primary server "
+                    "in %s seconds to ensure the backup can complete."
+                    % (times, self.primary_checkpoint_timeout)
+                )
+                sleep_time = datetime.datetime.now() + datetime.timedelta(
+                    seconds=self.primary_checkpoint_timeout
+                )
+                while True:
+                    try:
+                        # Always check if the queue is empty, so we know to stop
+                        # before the checkpoint execution
+                        if done_q.get(timeout=wait):
+                            return
+                    except Empty:
+                        # If the queue is empty, we can proceed to the checkpoint
+                        # if enough time has passed
+                        if sleep_time < datetime.datetime.now():
+                            self.primary.checkpoint()
+                            self.primary.switch_wal()
+                            break
+                            # break out of the loop after the checkpoint and wal switch
+                            # execution. The connection will be closed in the finally statement
         finally:
             # Close the connection since only this subprocess will ever use it
             self.primary.close()
@@ -1720,3 +1845,108 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
         return self._stop_backup(
             super(StandbyPostgreSQLConnection, self).stop_exclusive_backup
         )
+
+
+class PostgresKeepAlive:
+    """
+    Context manager to maintain a Postgres connection alive.
+
+    A child thread is spawned to execute heartbeat queries in the background
+    at a specified interval during its living context.
+
+    It does not open or close any connections on its own. Instead, it waits for the
+    specified connection to be opened on the main thread before start sending any query.
+
+    :cvar THREAD_NAME: The name identifying the keep-alive thread.
+    """
+
+    THREAD_NAME = "barman_keepalive_thread"
+
+    def __init__(self, postgres, interval, raise_exception=False):
+        """
+        Constructor.
+
+        :param barman.postgres.PostgreSQLConnection postgres: The
+            Postgres connection to keep alive.
+        :param int interval: An interval in seconds at which a
+            heartbeat query will be sent to keep the connection alive.
+            A value <= ``0`` won't start the keepalive.
+        :param bool raise_exception: A boolean indicating if an exception
+            should be raised in case the connection is lost. If ``True``, a
+            ``PostgresConnectionLost`` exception will be raised as soon as
+            it's noticed a connection failure. If ``False``, it will keep executing
+            normally until the context exits.
+        """
+        self.postgres = postgres
+        self.interval = interval
+        self.raise_exception = raise_exception
+        self._stop_thread = threading.Event()
+        self._thread = threading.Thread(
+            target=self._run_keep_alive,
+            name=self.THREAD_NAME,
+        )
+
+    def _prepare_signal_handler(self):
+        """
+        Set up a signal handler to raise an exception on the main thread when
+        the keep-alive thread wishes to interrupt it. This method listens for a
+        ``SIGUSR1`` signal and, when received, raises a ``PostgresConnectionLost``
+        exception.
+
+        .. note::
+            This code is, and only works if, executed while on the main thread. We
+            are not able to set a signal listener on a child thread, therefore this
+            method must be executed before the keep-alive thread starts.
+        """
+
+        def raise_exception(signum, frame):
+            raise PostgresConnectionLost("Connection to Postgres server was lost.")
+
+        signal.signal(signal.SIGUSR1, raise_exception)
+
+    def _raise_exception_on_main(self):
+        """
+        Trigger an exception on the main thread at whatever frame is being executed
+        at the moment. This is done by sending a ``SIGUSR1`` signal to the process,
+        which will be caught by the signal handler set previously in this class.
+
+        .. note::
+            This is an alternative way of interrupting the main thread's work, since
+            there is no direct way of killing or raising exceptions on the main thread
+            from a child thread in Python. A handler for this signal has been set
+            beforehand by the ``_prepare_signal_handler`` method in this class.
+        """
+        os.kill(os.getpid(), signal.SIGUSR1)
+
+    def _run_keep_alive(self):
+        """Runs the keepalive until a stop-thread event is set"""
+        while not self._stop_thread.is_set():
+            if not self.postgres.has_connection:
+                # Wait for the connection to be opened on the main thread
+                time.sleep(1)
+                continue
+
+            success, ex = self.postgres.send_heartbeat_query()
+
+            if not success and self.raise_exception:
+                # If one of the below exeptions was raised by psycopg2, it most likely
+                # means that the connection (and consequently, the session) was lost. In
+                # such cases, we can stop the keep-alive exection and raise the exception
+                if isinstance(ex, (psycopg2.InterfaceError, psycopg2.OperationalError)):
+                    self._stop_thread.set()
+                    self._raise_exception_on_main()
+
+            self._stop_thread.wait(self.interval)
+
+    def __enter__(self):
+        """Enters context. Starts the thread"""
+        if self.interval > 0:
+            if self.raise_exception:
+                self._prepare_signal_handler()
+            self._thread.start()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """Exits context. Makes sure the thread is terminated"""
+        if self.interval > 0:
+            self._stop_thread.set()
+            self._thread.join()
